/********************************* Class Source Code ***************************/
/**
\package  _uc_Uml - 
\class    UmlView (Build: 94)

\brief    UML View
          Displays the class hierarchy as UML Diagram using Graphviz

\date     $Date: 2011-10-26 13:55:08.34 $
\note     dbsource gui.dev - ODABA Version 11.2.0

          Copyright (C) 1994-2011 Run Software GmbH.
 
          This program is free software: you can redistribute it and/or modify
          it under the terms of the GNU General Public License as published by
          the Free Software Foundation, either version 3 of the License, or
          (at your option) any later version.
 
          This program is distributed in the hope that it will be useful,
          but WITHOUT ANY WARRANTY; without even the implied warranty of
          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
          GNU General Public License 3.0 for more details.
 
          You should have received a copy of the GNU General Public License
          along with this program. If not, see <http://www.gnu.org/licenses/>.

*/
/******************************************************************************/
#define    OBJ_ID  "UmlView"

#include  <h/p_uc_Uml.h>
#include  <h/cqt.h>
#include  <h/UmlView.moc>
#include  <h/cqt.h>
#include  <h/cqt.h>
#include  <h/UmlSettingsDialog.hpp>
#include  <h/UmlView.hpp>








/**************************************************************************//**
\brief  Initialize - 


******************************************************************************/
#undef     MOD_ID
#define    MOD_ID  "Initialize"

/**************************************************************************//**
\brief  Initialize - 
*******************************************************************************
\brief  default - 



*******************************************************************************
******************************************************************************/

void UmlView :: Initialize ( )
{

  uml          = new ImageWidget(this);
  error_label  = new QLabel(this);
  display      = new QWidget(this);
  source       = new QTextEdit(this);
  stack_layout = new QStackedLayout(this);
  settings     = new UmlSettingsDialog(this);

  /*0*/ stack_layout->addWidget(uml);
  /*1*/ stack_layout->addWidget(error_label);
  /*2*/ stack_layout->addWidget(source);
  /*3*/ stack_layout->addWidget(settings);
  display->setLayout(stack_layout);
  stack_layout->setCurrentIndex(0);

  error_label->setText("An error has occured.<br/>Please, review your Graphviz configuration<br/>and make sure 'dot' is in you PATH");

  QScrollArea::setWidget(display);
  QScrollArea::setBackgroundRole(QPalette::Dark);
  QScrollArea::setAlignment(Qt::AlignHCenter);
  
  SetupContextMenu();

}

/**************************************************************************//**
\brief  SetupContextMenu - 


******************************************************************************/
#undef     MOD_ID
#define    MOD_ID  "SetupContextMenu"

/**************************************************************************//**
\brief  SetupContextMenu - 
*******************************************************************************
\brief  default - 



*******************************************************************************
******************************************************************************/

void UmlView :: SetupContextMenu ( )
{
  QAction *act;
  QList<QAction*> actions;
  QList<QAction*>::iterator it;

  // remove actions from ogui
  actions  = QScrollArea::actions();
  for(it = actions.begin() ;it!= actions.end() ;it++ ){
     removeAction(*it);
  }

  // extend context_menu with save & print
  act = new QAction("Redraw", this);
  connect(act,SIGNAL(triggered()),this,SLOT(slotRedraw()));
  addAction(act);

  setContextMenuPolicy(Qt::ActionsContextMenu);
  act  = new QAction("Save", this);
  connect(act,SIGNAL(triggered()),this,SLOT(slotSave()));
  addAction(act);

  act = new QAction("Print", this);
  connect(act,SIGNAL(triggered()),this,SLOT(slotPrint()));
  addAction(act);

  act = new QAction("Source", this);
  connect(act,SIGNAL(triggered()),this,SLOT(slotSource()));
  addAction(act);

  act = new QAction("Settings", this);
  connect(act,SIGNAL(triggered()),this,SLOT(slotSettings()));
  addAction(act);


}

/**************************************************************************//**
\brief  UmlView - Constructor
        Initializes   the   UML   View   and   the   Image   Widget and connects
        

******************************************************************************/
#undef     MOD_ID
#define    MOD_ID  "UmlView"

/**************************************************************************//**
\brief  UmlView - Constructor
*******************************************************************************
\brief  default - 



\param  pUserControlHandle - 
\param  pQWidget - 
*******************************************************************************
******************************************************************************/

     UmlView :: UmlView (UserControlHandle *pUserControlHandle, ::QWidget *pQWidget )
                     :   QScrollArea(pQWidget)
 ,UserControl(pUserControlHandle)
 ,display(NULL)
 ,uml(NULL)
 ,error_label(NULL)
 ,settings(NULL)
 ,source(NULL)
 ,stack_layout(NULL)
{

  Initialize();


}

/**************************************************************************//**
\brief  drawDot - 


******************************************************************************/
#undef     MOD_ID
#define    MOD_ID  "drawDot"

/**************************************************************************//**
\brief  drawDot - 
*******************************************************************************
\brief  i0 - 



\param  qsValue - 
*******************************************************************************
******************************************************************************/

void UmlView :: drawDot (::QString qsValue )
{
  QString      cmd;
  QStringList  args;
  QByteArray   dotar;
  char        *dot;
  QProcess     p;

  source->setText(qsValue);
  cmd="dot";
  args << "-Tpng";
  p.start(cmd,args);
  if(!p.waitForStarted()){
    error_label->setText("dot cannot be found<br/>dot is part of GraphViz<br/>GraphViz must be in your PATH<br/>Please install GraphViz<br/>and expand your Environment");
    stack_layout->setCurrentIndex(1);
    return;
  }
  dotar = qsValue.toUtf8();
  dot = dotar.data();
  p.write(dot);
  p.closeWriteChannel();
  if(!p.waitForFinished()){
    error_label->setText("dot cannot render the source.<br/>review the generated source!");
    stack_layout->setCurrentIndex(1);
    return;
  }
  uml->get_image().load(&p,"PNG");
  uml->resize(uml->get_image().size());
  uml->setMinimumSize(uml->get_image().size());
  uml->update();
  if(stack_layout->currentIndex()==0){
    display->setMinimumSize(uml->get_image().size());
    display->setMaximumSize(uml->get_image().size());
  }
  QScrollArea::setAlignment(Qt::AlignHCenter);


}

/**************************************************************************//**
\brief  fillData - fillData event
        handles  the  event  that  ogui  requests  the control to reconsider the
        contents.
        Reads   the    propertyhandle   and   executes   GetDOTUML   in   the  
        resourceDatabase.
        Configures the DOTml to draw the directed graph like UML requests.
        Executes  'dot'  and  loads  the  result  as  PNG image and triggers the
        update of the control

******************************************************************************/
#undef     MOD_ID
#define    MOD_ID  "fillData"

/**************************************************************************//**
\brief  fillData - fillData event
*******************************************************************************
\brief  i0 - 



*******************************************************************************
******************************************************************************/

void UmlView :: fillData ( )
{
  odaba::Property prop;
  QString         qdot;
  std::string     dot, edot, thisclass;
  odaba::Property res; 

  if ( !controlHandle() || !controlHandle()->property()->isValid() ){
    error_label->setText("The provided DataHandle is not valid");
    stack_layout->setCurrentIndex(1);
    return;
  }
  prop.open(*controlHandle()->property());
  try {
    thisclass = static_cast<const char *>(prop.value("sys_ident").toString().area());
    res = prop.executeExpression(
            odaba::Application::resourceDatabase()
          , "GetDOTUML()");
    edot  = "\nfontname = \"Sans\"";
    edot += "\nfontsize = 8";
    if(odaba::Option("Options.Documentation.Graph.Size").toString().toUTF8().data()){
      edot += "\nsize=\"";
      edot += odaba::Option("Options.Documentation.Graph.Size").toString().toUTF8().data();
      edot += "\"";
    }
    edot += "\nnode [";
    edot += "\n fontname = \"Sans\"";
    edot += "\n fontsize = 8";
    edot += "\n shape = record";
    edot += "\n]";
    edot += "\nedge [";
    edot += "\n fontname = \"Sans\"";
    edot += "\n fontsize = 8";
    edot += "\n]";
    edot += "\n";
    edot += static_cast<const char *>(res.value().toString().area());
    edot += "\n label= \"uml for: ";
    edot += thisclass;
    edot += "\";";
  } catch (odaba::Exception &e){
    edot = "\n label = \"Exception processing expression:";
    edot+= e.what();
    edot+="\";";
  } catch (...) {
    edot = "\n label = \"Exception processing expression\";";
  }

  dot="digraph uml { ";
  dot += edot;
  dot+="\n}";
  qdot = dot.c_str();
  drawDot(qdot);


}

/**************************************************************************//**
\brief  slotPrint - Print
        Prints the loaded image using the QPrinter object

******************************************************************************/
#undef     MOD_ID
#define    MOD_ID  "slotPrint"

/**************************************************************************//**
\brief  slotPrint - Print
*******************************************************************************
\brief  default - 



*******************************************************************************
******************************************************************************/

void UmlView :: slotPrint ( )
{

#ifndef QT_NO_PRINTER
  QPrinter printer(QPrinter::HighResolution);
  QPrintDialog *printDialog = new QPrintDialog(&printer, this);
  if (printDialog->exec() != QDialog::Accepted)      return;
  QPainter painter(&printer);
  QRect rect = painter.viewport();
  QSize size = uml->get_image().size();
  size.scale(rect.size(), Qt::KeepAspectRatio);
  painter.setViewport(rect.x(), rect.y(), size.width(), size.height());
  painter.setWindow(uml->get_image().rect());
  painter.drawImage(0, 0, uml->get_image());
#endif // QT_NO_PRINTER


}

/**************************************************************************//**
\brief  slotRedraw - 


******************************************************************************/
#undef     MOD_ID
#define    MOD_ID  "slotRedraw"

/**************************************************************************//**
\brief  slotRedraw - 
*******************************************************************************
\brief  default - 



*******************************************************************************
******************************************************************************/

void UmlView :: slotRedraw ( )
{
  QString qdot;
  stack_layout->setCurrentIndex(0);
  qdot = source->toPlainText();
  drawDot(qdot);

}

/**************************************************************************//**
\brief  slotSave - Save
        Saves the current image as PNG

******************************************************************************/
#undef     MOD_ID
#define    MOD_ID  "slotSave"

/**************************************************************************//**
\brief  slotSave - Save
*******************************************************************************
\brief  default - 



*******************************************************************************
******************************************************************************/

void UmlView :: slotSave ( )
{
  QString fname;
  QFile   f;
  fname = QFileDialog::getSaveFileName(this,"Save the UML as PNG");
  f.setFileName(fname);
  if(!f.open(QIODevice::WriteOnly)){
    error_label->setText("could not open filehandle. file NOT saved");
    stack_layout->setCurrentIndex(1);
    return;
  }

  uml->get_image().save(&f,"PNG");
  f.close();


}

/**************************************************************************//**
\brief  slotSettings - 


******************************************************************************/
#undef     MOD_ID
#define    MOD_ID  "slotSettings"

/**************************************************************************//**
\brief  slotSettings - 
*******************************************************************************
\brief  default - 



*******************************************************************************
******************************************************************************/

void UmlView :: slotSettings ( )
{

  display->setMinimumSize(QSize(400,400));
  display->setMaximumSize(QSize(400,400));
  stack_layout->setCurrentIndex(3);

}

/**************************************************************************//**
\brief  slotSource - 


******************************************************************************/
#undef     MOD_ID
#define    MOD_ID  "slotSource"

/**************************************************************************//**
\brief  slotSource - 
*******************************************************************************
\brief  default - 



*******************************************************************************
******************************************************************************/

void UmlView :: slotSource ( )
{

  display->setMinimumSize(QSize(400,display->parentWidget()->height()));
  display->setMaximumSize(QSize(400,display->parentWidget()->height()));
  stack_layout->setCurrentIndex(2);

}

/**************************************************************************//**
\brief  widget - Widget
        Returns the QWidget for the Usercontrol

\return pQWidget - 
******************************************************************************/
#undef     MOD_ID
#define    MOD_ID  "widget"

/**************************************************************************//**
\brief  widget - Widget
*******************************************************************************
\brief  i0 - 



*******************************************************************************
\return pQWidget - 
******************************************************************************/

::QWidget *UmlView :: widget ( )
{

  return dynamic_cast<QWidget*>(dynamic_cast<QScrollArea*>(this));


}

/**************************************************************************//**
\brief  ~UmlView - 


******************************************************************************/
#undef     MOD_ID
#define    MOD_ID  "~UmlView"

/**************************************************************************//**
\brief  ~UmlView - 
*******************************************************************************
\brief  default - 



*******************************************************************************
******************************************************************************/

     UmlView :: ~UmlView ( )
{



}


