/********************************* Class Source Code ***************************/
/**
\package  _uc_WebKitControl - 
\class    WebKitControl (Build: 20)

\brief    


\date     $Date: 2011-10-26 14:17:26.64 $
\note     dbsource gui.dev - ODABA Version 11.2.0

          Copyright (C) 1994-2011 Run Software GmbH.
 
          This program is free software: you can redistribute it and/or modify
          it under the terms of the GNU General Public License as published by
          the Free Software Foundation, either version 3 of the License, or
          (at your option) any later version.
 
          This program is distributed in the hope that it will be useful,
          but WITHOUT ANY WARRANTY; without even the implied warranty of
          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
          GNU General Public License 3.0 for more details.
 
          You should have received a copy of the GNU General Public License
          along with this program. If not, see <http://www.gnu.org/licenses/>.

*/
/******************************************************************************/
#define    OBJ_ID  "WebKitControl"

#include  <h/p_uc_WebKitControl.h>
#include  <h/WebKitControl.moc>
#include  <h/cqt.h>
#include  <h/cqt.h>
#include  <h/WebKitPage.hpp>
#include  <h/WebKitControl.hpp>





/**************************************************************************//**
\brief  WebKitControl - 


******************************************************************************/
#undef     MOD_ID
#define    MOD_ID  "WebKitControl"

/**************************************************************************//**
\brief  WebKitControl - 
*******************************************************************************
\brief  default - 



\param  pUserControlHandle - 
\param  parent - 
\param  cControlData - 
*******************************************************************************
******************************************************************************/

     WebKitControl :: WebKitControl (UserControlHandle *pUserControlHandle, ::QWidget *parent, const char *cControlData )
                     :   QObject (parent),
  UserControl (pUserControlHandle),
  data(cControlData)

{
  QStringList dataitems;
  webView= new QWebView( parent );
  webView->setPage(new WebKitPage(this));
  webView->setFocusPolicy( Qt::WheelFocus );
  
  QAction *reload = new QAction("reload",this);
  reload->setShortcut(QKeySequence("Ctrl+R"));
  connect(reload,SIGNAL(triggered()),this,SLOT(refresh()));
  webView->addAction(reload);

  connect(webView->page()->mainFrame()
         ,SIGNAL(javaScriptWindowObjectCleared())
         ,this
         ,SLOT(setup())
         );
  connect(webView
         ,SIGNAL(loadFinished(bool))
         ,this
         ,SLOT(loaded())
         );
  dataitems = data.split("\n");  
  resource_url = dataitems[0];

}

/**************************************************************************//**
\brief  fillData - Fill Data Event
        Load the URL from the style-options.

******************************************************************************/
#undef     MOD_ID
#define    MOD_ID  "fillData"

/**************************************************************************//**
\brief  fillData - Fill Data Event
*******************************************************************************
\brief  i0 - 



*******************************************************************************
******************************************************************************/

void WebKitControl :: fillData ( )
{

  webView->load(resource_url);


}

/**************************************************************************//**
\brief  loaded - Evaluate Property Data and Send to Javascript
        Reads  the data from  the associated Property  and sends its value(s) as
        string using the 'setControlData' signal.

******************************************************************************/
#undef     MOD_ID
#define    MOD_ID  "loaded"

/**************************************************************************//**
\brief  loaded - Evaluate Property Data and Send to Javascript
*******************************************************************************
\brief  default - 



*******************************************************************************
******************************************************************************/

void WebKitControl :: loaded ( )
{
  odaba::Property     property(*control_handle->property());
  QString             qsdata;
BEGINSEQ
  if (!property.isValid() )                   ERROR
  if ( property.count() > 1 ){
    // handle collection
    try {
      qsdata = "[";
      property.top();
      while ( property.next() ){
        if ( property.currentPosition()!=0 )
          qsdata+= ", ";
        qsdata+=property.value().toString().data();
      }
      qsdata+= "]";
    } catch ( odaba::Exception e ){
      emit infoException(e.what());
      return;
    }
  } else {
    // handle instance
    qsdata = property.value().toString().data();
  }
  emit setControlData(qsdata);

RECOVER

ENDSEQ

}

/**************************************************************************//**
\brief  message - Send Message to Control Context
        Sends  a  Message  to  the  ControlContext that can be handled using the
        doMessage                   Handler.                  
        

******************************************************************************/
#undef     MOD_ID
#define    MOD_ID  "message"

/**************************************************************************//**
\brief  message - Send Message to Control Context
*******************************************************************************
\brief  default - 



\param  qsMessage - 
*******************************************************************************
******************************************************************************/

void WebKitControl :: message (const ::QString &qsMessage )
{

BEGINSEQ
if ( !control_handle )                              ERROR
control_handle->message(qsMessage.toUtf8().data());
RECOVER

ENDSEQ

}

/**************************************************************************//**
\brief  refresh - Reload the Webkit Page
        reload the url. (associated to Ctrl+R) 


******************************************************************************/
#undef     MOD_ID
#define    MOD_ID  "refresh"

/**************************************************************************//**
\brief  refresh - Reload the Webkit Page
*******************************************************************************
\brief  default - 



*******************************************************************************
******************************************************************************/

void WebKitControl :: refresh ( )
{

fillData();

}

/**************************************************************************//**
\brief  setup - Setup the Javascript Object
        Add the wkc Object to the Javascript Window Object (internal) 


******************************************************************************/
#undef     MOD_ID
#define    MOD_ID  "setup"

/**************************************************************************//**
\brief  setup - Setup the Javascript Object
*******************************************************************************
\brief  default - 



*******************************************************************************
******************************************************************************/

void WebKitControl :: setup ( )
{

  webView->page()->mainFrame()
    ->addToJavaScriptWindowObject("wkc", this);

}

/**************************************************************************//**
\brief  widget - 


\return pQWidget - 
******************************************************************************/
#undef     MOD_ID
#define    MOD_ID  "widget"

/**************************************************************************//**
\brief  widget - 
*******************************************************************************
\brief  i0 - 



*******************************************************************************
\return pQWidget - 
******************************************************************************/

::QWidget *WebKitControl :: widget ( )
{

  return webView;

}

/**************************************************************************//**
\brief  ~WebKitControl - 


******************************************************************************/
#undef     MOD_ID
#define    MOD_ID  "~WebKitControl"

/**************************************************************************//**
\brief  ~WebKitControl - 
*******************************************************************************
\brief  default - 



*******************************************************************************
******************************************************************************/

     WebKitControl :: ~WebKitControl ( )
{



}


